package com.hz.pm.api.projectdeclared.utils;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.service.IProjectService;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * <p>
 * ProjectIdCodeCacheUtil
 * </p>
 *
 * @author WendyYang
 * @since 18:40 2024/3/25
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectIdCodeCacheUtil {

    @Data
    private static class IdCodeCacheDTO {

        private Long projectId;

        private String projectCode;

        private LocalDateTime createOn;

        private Boolean newest;

    }

    private final IProjectService projectService;

    @PostConstruct
    public void initProjectIdCodeCache() {
        Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getProjectCode, Project::getId, Project::getCreateOn, Project::getNewest);
        for (Project project : projectService.list(query)) {
            put(project.getId(), project);
        }
        log.info("初始化项目ID编码缓存：{}", CACHE.size());
    }

    //==================================================================================================================

    private static final Map<Long, IdCodeCacheDTO> CACHE = new ConcurrentHashMap<>();

    public static String get(Long projectId) {
        return Optional.ofNullable(CACHE.get(projectId))
                .flatMap(w -> Optional.of(w.getProjectCode()))
                .orElse(null);
    }

    public static List<Long> get(String projectCode) {
        return CACHE.entrySet()
                .stream()
                .filter(w -> StrUtil.equals(w.getValue().getProjectCode(), projectCode))
                .map(Map.Entry::getKey)
                .collect(Collectors.toList());
    }

    public static void put(Long projectId, Project project) {
        IdCodeCacheDTO idCodeCache = new IdCodeCacheDTO();
        idCodeCache.setProjectId(project.getId());
        idCodeCache.setProjectCode(project.getProjectCode());
        idCodeCache.setCreateOn(project.getCreateOn());
        idCodeCache.setNewest(project.getNewest());
        CACHE.put(projectId, idCodeCache);
    }

    public static Long newest(String projectCode) {
        return get(projectCode).stream().map(CACHE::get)
                .filter(IdCodeCacheDTO::getNewest)
                .findFirst()
                .flatMap(w -> Optional.of(w.getProjectId()))
                .orElse(null);
    }

    public static Long newest(Long projectId) {
        String projectCode = get(projectId);
        if (StrUtil.isBlank(projectCode)) {
            return projectId;
        }
        return newest(projectCode);
    }

}
